@@ -16,7 +16,7 @@
 
 """XmlEditor module"""
 
-__revision__ = "$Id: XmlEditor.py,v 1.10 2001/10/09 11:16:29 alf Exp $"
+__revision__ = "$Id: XmlEditor.py,v 1.14 2002/07/19 16:16:51 alf Exp $"
 
 #
 # NOTE: this file will need rewriting when XMLSchema is available
@@ -30,6 +30,15 @@ from xml.parsers.xmlproc.xmldtd import CompleteDTD
 from xml.dom.ext import ReleaseNode
 from xml.dom.ext.reader import Sax2
 from xml.dom.ext import StripXml, Print, PrettyPrint
+try:
+    from xml.dom import EMPTY_NAMESPACE as NO_NS
+except:
+    NO_NS = ''
+
+try:
+    from Ft.Xml.Xpath import Evaluate
+except:
+    from xml.xpath import Evaluate
 
 def parse_dtd_file(dtd_file,dtd_obj=None):
     parser = DTDParser()
@@ -39,6 +48,24 @@ def parse_dtd_file(dtd_file,dtd_obj=None):
     parser.parse_resource(dtd_file)
     parser.deref()
     return dtd
+
+def prefix_to_nsuri(node):
+    map = {}
+    ns_nodes = Evaluate('namespace::node()',node)
+    for ns in ns_nodes:
+        map[ns.localName]=ns.value
+    return map
+
+def get_prefix(qname):
+    splitted = qname.split(':',1)
+    if len(splitted) == 1:
+        return None
+    else:
+        return splitted[0]
+    
+def get_uri(node,prefix):
+    map = prefix_to_nsuri(node)
+    return  map.get(prefix)
     
 def getElementsName(child,dtd,list=None):
     """
@@ -261,9 +288,10 @@ class XmlEditor(GtkVBox) :
                 popup.popup(None,None,None,3,event.time)
 
     def add_attribute_activated(self,item,xmlnode,treenode,attribute):
-        if not xmlnode.hasAttributeNS('',attribute.get_name()):
+        uri = get_uri(xmlnode,get_prefix(attribute.get_name()))
+        if not xmlnode.hasAttributeNS(uri,attribute.get_name()):
             value = attribute.get_default() or ''
-            xmlnode.setAttributeNS('',attribute.get_name(),value)
+            xmlnode.setAttributeNS(uri,attribute.get_name(),value)
         
         
     def delete_child_activated(self,item,xmlnode,treenode):
@@ -280,7 +308,8 @@ class XmlEditor(GtkVBox) :
             new_element = xmlnode.ownerDocument.createTextNode('')
             isleaf=1
         else:
-            new_element = xmlnode.ownerDocument.createElementNS('',tag)
+            uri = get_uri(xmlnode,get_prefix(tag))
+            new_element = xmlnode.ownerDocument.createElementNS(uri,tag)
             isleaf=0
             
         if position < 0 or position >= len(xmlnode.childNodes):
